import {Component, Inject, Input, OnInit} from '@angular/core';
import {MAT_DIALOG_DATA, MatDialog, MatDialogRef} from '@angular/material/dialog';
import { HttpClient } from '@angular/common/http';
import { FormBuilder, FormGroup } from '@angular/forms';
import {Router} from "@angular/router";
import {ToastrService} from "ngx-toastr";

@Component({
  selector: 'app-execute-command',
  templateUrl: './execute-command.component.html',
  styleUrls: ['./execute-command.component.css']
})
export class ExecuteCommandComponent implements OnInit {
  @Input() clientData: any = {};
  baseUrl: string = import.meta.env.NG_APP_BASE_API_URL;
  loading: boolean = true;

  arrayCommands: any[] = [
    {name: 'Enceder', slug: 'power-on', disabled: false},
    {name: 'Apagar', slug: 'power-off', disabled: false},
    {name: 'Reiniciar', slug: 'reboot', disabled: false},
    {name: 'Iniciar Sesión', slug: 'login', disabled: true},
    {name: 'Crear Image', slug: 'create-image', disabled: false},
    {name: 'Deploy Image', slug: 'deploy-image', disabled: false},
    {name: 'Eliminar Imagen Cache', slug: 'delete-image-cache', disabled: true},
    {name: 'Particionar y Formatear', slug: 'partition', disabled: false},
    {name: 'Inventario Software', slug: 'software-inventory', disabled: true},
    {name: 'Inventario Hardware', slug: 'hardware-inventory', disabled: true},
    {name: 'Ejecutar script', slug: 'run-script', disabled: true},
  ];

  constructor(
    private dialog: MatDialog,
    private http: HttpClient,
    private fb: FormBuilder,
    private router: Router,
    private toastService: ToastrService
  ) {
  }

  ngOnInit(): void {
    this.clientData = this.clientData || {};
    this.loadClient(this.clientData)
  }

  loadClient = (uuid: string) => {
    this.http.get<any>(`${this.baseUrl}${uuid}`).subscribe({
      next: data => {
        this.clientData = data;
        this.loading = false;
      },
      error: error => {
        console.error('Error al obtener el cliente:', error);
      }
    });
  }

  onCommandSelect(action: any): void {
    if (action === 'partition') {
      this.openPartitionAssistant();
    }

    if (action === 'create-image') {
      this.openCreateImageAssistant();
    }

    if (action === 'deploy-image') {
      this.openDeployImageAssistant();
    }

    if (action === 'reboot') {
      this.rebootClient();
    }

    if (action === 'power-off') {
      this.powerOffClient();
    }

    if (action === 'power-on') {
      this.powerOnClient();
    }
  }

  rebootClient(): void {
    this.http.post(`${this.baseUrl}/clients/server/${this.clientData.uuid}/reboot`, {}).subscribe(
      response => {
        this.toastService.success('Cliente actualizado correctamente');
      },
      error => {
        this.toastService.error('Error de conexión con el cliente');
      }
    );
  }

  powerOnClient(): void {
    const payload = {
      client: this.clientData['@id']
    }

    this.http.post(`${this.baseUrl}${this.clientData.repository['@id']}/wol`, payload).subscribe(
      response => {
        this.toastService.success('Cliente actualizado correctamente');
      },
      error => {
        this.toastService.error('Error de conexión con el cliente');
      }
    );
  }

  powerOffClient(): void {
    this.http.post(`${this.baseUrl}/clients/server/${this.clientData.uuid}/power-off`, {}).subscribe(
      response => {
        this.toastService.success('Cliente actualizado correctamente');
      },
      error => {
        this.toastService.error('Error de conexión con el cliente');
      }
    );
  }

  openPartitionAssistant(): void {
    this.router.navigate([`/clients/${this.clientData.uuid}/partition-assistant`]).then(r => {
      console.log('navigated', r);
    });
  }

  openCreateImageAssistant(): void {
    this.router.navigate([`/clients/${this.clientData.uuid}/create-image`]).then(r => {
      console.log('navigated', r);
    });
  }

  openDeployImageAssistant(): void {
    this.router.navigate([`/clients/${this.clientData.uuid}/deploy-image`]).then(r => {
      console.log('navigated', r);
    });
  }

}
