import {Component, Inject, OnInit, Optional} from '@angular/core';
import {MatDialogRef} from "@angular/material/dialog";
import {HttpClient} from "@angular/common/http";
import {MatSnackBar} from "@angular/material/snack-bar";
import {ToastrService} from "ngx-toastr";

@Component({
  selector: 'app-create-multiple-client',
  templateUrl: './create-multiple-client.component.html',
  styleUrl: './create-multiple-client.component.css'
})
export class CreateMultipleClientComponent implements OnInit{
  baseUrl: string = import.meta.env.NG_APP_BASE_API_URL;
  parentUnits: any[] = [];
  uploadedClients: any[] = [];
  loading: boolean = false;
  displayedColumns: string[] = ['name', 'ip', 'mac'];
  showTextarea: boolean = true;
  organizationalUnit: any;

  constructor(
    @Optional() private dialogRef: MatDialogRef<CreateMultipleClientComponent>,
    private http: HttpClient,
    private snackBar: MatSnackBar,
    private toastService: ToastrService
  ) {}

  ngOnInit(): void {
    this.loadParentUnits();
  }

  loadParentUnits(): void {
    this.loading = true;
    const url = `${this.baseUrl}/organizational-units?page=1&itemsPerPage=10000`;

    this.http.get<any>(url).subscribe(
      response => {
        this.parentUnits = response['hydra:member'];
        this.loading = false;
      },
      error => {
        console.error('Error fetching parent units:', error);
        this.loading = false;
      }
    );
  }

  setOrganizationalUnit(organizationalUnit: any): void {
    console.log('Organizational unit selected:', organizationalUnit.value);
    this.organizationalUnit = organizationalUnit.value;
  }

  onFileUpload(event: any): void {
    const file = event.target.files[0];
    if (file) {
      const reader = new FileReader();

      reader.onload = (e: any) => {
        const textData = e.target.result;
        const regex = /host\s+(\S+)\s+\{\s+hardware\s+ethernet\s+([\da-fA-F:]+);\s+fixed-address\s+([\d.]+);\s+\}/g;
        let match;
        const clients = [];

        while ((match = regex.exec(textData)) !== null) {
          clients.push({
            name: match[1],
            mac: match[2],
            ip: match[3]
          });
        }

        if (clients.length > 0) {
          this.uploadedClients = clients;
          this.toastService.success('Archivo cargado correctamente, los datos están listos para enviarse.', 'Éxito');
          this.showTextarea = false;
        } else {
          this.toastService.error('No se encontraron datos válidos', 'Error');
          this.showTextarea = true;
        }
      };

      reader.readAsText(file);
    }
  }

  onTextarea(text: string): void {
    const regex = /host\s+(\S+)\s+\{\s+hardware\s+ethernet\s+([\da-fA-F:]+);\s+fixed-address\s+([\d.]+);\s+\}/g;
    let match;
    const clients = [];

    while ((match = regex.exec(text)) !== null) {
      clients.push({
        name: match[1],
        mac: match[2],
        ip: match[3]
      });
    }

    if (clients.length > 0) {
      this.uploadedClients = clients;
      this.toastService.success('Datos cargados correctamente, los datos están listos para enviarse.', 'Éxito');
      this.showTextarea = false;
    } else {
      this.toastService.error('No se encontraron datos válidos', 'Error');
      this.showTextarea = true;
    }
  }

  onSubmit(): void {
    if (this.uploadedClients.length > 0) {
      this.uploadedClients.forEach(client => {
        const formData = {
          organizationalUnit: this.organizationalUnit,
          name: client.name || null,
          mac: client.mac || null,
          ip: client.ip || null,
        };

        this.http.post(`${this.baseUrl}/clients`, formData).subscribe(
          response => {
            this.toastService.success(`Cliente ${client.name} creado exitosamente`, 'Éxito');
          },
          error => {
            this.toastService.error(error.error['hydra:description'], `Error al crear el cliente ${client.name}`);
          }
        );
      });
      this.uploadedClients = [];
      this.dialogRef.close();
    } else {
      this.toastService.error('No hay clientes cargados para añadir', 'Error');
    }
  }

  onNoClick(): void {
    this.dialogRef.close();
  }
}
